/*
 * CinemAsie.java, 17 févr. 2006
 * 
 * This file is part of FilmZ.
 *
 * Copyright © 2006 Johan Cwiklinski
 *
 * File :               CinemAsie.java
 * Author's email :     johan@x-tnd.be
 * Author's Website :   http://www.x-tnd.be/
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 */

/**
 * 
 */
package be.xtnd.films.plugins;

import java.util.regex.Pattern;

/**
 * @author Johan Cwiklinski
 * @version 1.0
 */
public class CinemAsie extends RecupWeb{
    
	/**
	 * Cherche le titre en paramètre
	 * sur cinemasie.com
	 * 
	 * @param title titre à chercher
	 */
	public CinemAsie(String title){
		super(
				title,
				"http://www.cinemasie.com/fr/recherche.html?texte=[word]&tous=1",
				"tous les titres \\((.*?)\\)</td>",
				false
		);
	}
	
	/**
	 * Cherche et traite si besoin est
	 * les pages suivantes des résultats 
	 * 
	 * @param data Données de la page en cours pour recherche du lien
	 * vers la page suivante
	 */
	protected void nextPages(StringBuffer data){
    	/*boolean isNext = false;
		REGEX = "<a href=[^\\\"]*\\\"(/recherche.*?)\\\"[^>]*>Films suivants</a>";
    	pattern = Pattern.compile(REGEX);
    	matcher = pattern.matcher(data);
    	while(matcher.find()){	
    		//System.out.println(matcher.groupCount()+matcher.group(1));
    		//System.out.println("new page found, parsing...");
    		data = loadUrl("http://www.allocine.fr"+matcher.group(1));
    		parseResults(data);
    		isNext=true;
    	}		
		if(isNext) nextPages(data);*/
	}

	/**
	 * Parcours les résultats de la liste et les stocke
	 * dans un Object[][] pour l'affichage ultérieur dans
	 * une JTable
	 * 
	 * @param data données à parser
	 */
	protected void parseResults(StringBuffer data){
		REGEX = "<tr  class=\"ligneliste-c[0|1]\"><td class=\"indpic\">([0-9]{4})</td>*.*?<td><a href=\"(.*?)\" title=\"fiche du film\">(.*?)</a></td>*.*?<a href=\".*?\">(.*?)</a>*.*?</tr>";
		pattern = Pattern.compile(REGEX,Pattern.DOTALL);
        matcher = pattern.matcher(data);
        while(matcher.find()) {
    		listeReponses[current] = new Object[3];
    		String year = matcher.group(1);
    		String rec_url = matcher.group(2);
    		String rec_fr_title = matcher.group(4).trim();
    		String rec_en_title = matcher.group(3).trim();

    		listeReponses[current][2] = year;
    		listeReponses[current][1] = rec_en_title+" ("+rec_fr_title+")";
			listeReponses[current][0] = rec_url;
			
			current++;
        }
	}

	/**
	 * Charge la page des détails d'un film
	 * 
	 * @param url page du film à charger, récupérée normalement
	 * dans la liste des résultats de recherche
	 */
	public void loadMovie(String url){
		StringBuffer data = loadUrl("http://www.cinemasie.com"+url);
		System.out.println("http://www.cinemasie.com"+url);
		String titre_fr="", titre_eng="";
		//titre eng
		REGEX = "<div class=\"titrecol\">Titre Anglais</div>*.*?<td>(.*?)</td>";
		pattern = Pattern.compile(REGEX, Pattern.DOTALL);
		matcher = pattern.matcher(data);
		while(matcher.find())
			titre_eng = matcher.group(1).trim();
		
		//titre fr
		REGEX ="<div class=\"titrecol\">Titre Français</div>*.*?<td>(.*?)</td>";
		pattern = Pattern.compile(REGEX);
		matcher = pattern.matcher(data);

		while(matcher.find())
			titre_fr = matcher.group(1).trim();
		
		if(!titre_fr.equals("")){
			titre = titre_fr;
			titreOriginal = titre_eng;
		}else{
			titre = titre_eng;
		}
		
		//réalisateurs
		REGEX = "<div class=\"titrecol\">réalisateur</div>*.*?<div class=\"titrecol\">[a-z]{1}.*?</div>";
		pattern = Pattern.compile(REGEX, Pattern.DOTALL);
		matcher = pattern.matcher(data);
		while(matcher.find()){
			REGEX = "<a href=\"/fiche/personne/.*?\"[^>]*>(.*?)</a>";
			pattern = Pattern.compile(REGEX);
			matcher_second = pattern.matcher(matcher.group(0));
			while(matcher_second.find()){
				String[] temp = matcher_second.group(1).split(" - &#"); //supression nom chinois
				realisateurs.add(temp[0].trim());
			}

			REGEX = "<td>([a-z]{1}.*?)</td>";
			pattern = Pattern.compile(REGEX, Pattern.CASE_INSENSITIVE);
			matcher_second = pattern.matcher(matcher.group(0));
			while(matcher_second.find()){
				String[] temp = matcher_second.group(1).split(" - &#"); //supression nom chinois
				realisateurs.add(temp[0].trim());
			}
		}
		
		//acteurs
		REGEX = "<div class=\"titrecol\">interprète</div>*.*?<div class=\"titrecol\">[a-z]{1}.*?</div>";
		pattern = Pattern.compile(REGEX, Pattern.DOTALL);
		matcher = pattern.matcher(data);
		while(matcher.find())
			loadActors(matcher.group(0));
		
		//origines
		REGEX = "<div class=\"titrecol\">Pays</div>*.*?</tr>";
		pattern = Pattern.compile(REGEX,Pattern.DOTALL);
		matcher = pattern.matcher(data);
		while(matcher.find()){
			REGEX = "<a[^>]*>(.*?)</a>";
			pattern = Pattern.compile(REGEX);
			matcher_second = pattern.matcher(matcher.group(0));
			while(matcher_second.find())
				origines.add(matcher_second.group(1));
		}
		
		//genres
		REGEX = "<div class=\"titrecol\">Genre</div>*.*?</tr>";
		pattern = Pattern.compile(REGEX, Pattern.DOTALL);
		matcher = pattern.matcher(data);
		while(matcher.find()){
			REGEX = "<a[^>]*>(.*?)</a>";
			pattern = Pattern.compile(REGEX);
			matcher_second = pattern.matcher(matcher.group(0));
			while(matcher_second.find())
				genres.add(matcher_second.group(1));
		}
		
		/*
		 *  <div class="titrecol">Durée</div></td><td>([0-9])h([0-9]{1,2})</td>
		 *  <div class="titrecol">Durée</div></td><td>(([0-9]{1,3}) min.*?)</td>
		 *  */
		
		//durée
		REGEX = "<div class=\"titrecol\">Durée</div></td><td>([0-9])h([0-9]{1,2})</td>";
		pattern = Pattern.compile(REGEX, Pattern.CASE_INSENSITIVE);
		matcher = pattern.matcher(data);
		while(matcher.find()){ //durée 0h00
			duree = new Integer(matcher.group(1))*60+new Integer(matcher.group(2));
		}
		
		if(duree==0){//durée minutes
			REGEX = "<div class=\"titrecol\">Durée</div></td><td>([0-9]{1,3}).*?</td>";
			pattern = Pattern.compile(REGEX);
			matcher = pattern.matcher(data);
			while(matcher.find()){
				duree = new Integer(matcher.group(1));
			}
		}
		
		//année de production
		REGEX = "\\?annee=([0-9]{4})";
		pattern = Pattern.compile(REGEX);
		matcher = pattern.matcher(data);
		while(matcher.find())
			anneeProduction = new Integer(matcher.group(1));
		
		//année de sortie
		// inexistante sur ce site
		
		//synopsis
		//vérifier l'existance
		boolean syn_exists = false;
		REGEX = "last changed"; //le texte du wiki est parsé en anglais...
		pattern = Pattern.compile(REGEX);
		matcher = pattern.matcher(data);
		while(matcher.find())
			syn_exists = true;

		if(syn_exists){
			REGEX = "<div class=\"text-body\">(.*?)</div>";
			pattern = Pattern.compile(REGEX, Pattern.DOTALL);
			matcher = pattern.matcher(data);
			while(matcher.find())
				synopsis = matcher.group().replaceAll(rmMarks, "").trim();
		}
		
		System.out.println(data);
	}
	
	/**
	 * Charge la page de casting du film,
	 * récupère et stocke les acteurs dans un 
	 * ArrayList
	 *  
	 * @param url url de la page casting
	 */
	protected void loadActors(String url){
		REGEX = "<a href=\"/fiche/personne/.*?\"[^>]*>(.*?)</a>";
		pattern = Pattern.compile(REGEX);
		matcher = pattern.matcher(url);
		while(matcher.find()){
			String[] temp = matcher.group(1).split(" - &#"); //supression nom chinois
			acteurs.add(temp[0].trim());
		}
		
		REGEX = "<td>([a-z]{1}.*?)</td>";
		pattern = Pattern.compile(REGEX, Pattern.CASE_INSENSITIVE);
		matcher = pattern.matcher(url);
		while(matcher.find()){
			String[] temp = matcher.group(1).split(" - &#"); //supression nom chinois
			acteurs.add(temp[0].trim());
		}

		acteurs.trimToSize();
	}	
}
